/* (2,k)-cuckoo hashing su interi.
 *  
 * Tavola di hash con inserimento "stile cuculo" su interi positivi.
 * Utilizza d funzioni di hash e p elementi per bucket.
 *
 * Le funzioni di hash sono del tipo 
 * h(x) = ((ax + b) mod p) mod N se opera su N secchi.
 *
 * Alessio Orlandi
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


typedef struct
{
   char *chiave; /* Stringa. */
   int frequenza; /* Sua frequenza. */
} chiave_val;

typedef struct {
   /* Array di P elementi. */
   chiave_val *contenuto;
   /* Quanti sono gia' occupati (si mantiene
    * l'invariante che siano [0,..P-1].
    * Quanto occupati = 0 contenuto e' NULL. */
   int occupati;
} secchio;

typedef struct {
  int P; /* Numero di elementi per secchio. */

  /* Numero di secchi. */
  int nsecchi;

  /* Array di nsecchi puntatori ai secchi. */
  secchio *tavola;

  /* Numero di elementi in totale. */
  int carico;

  /* Numero di collisioni. */
  int ncollisioni; 

} hash;

void inizializza ( hash *h, int nsecchi, int P)
{
  int i;

  h->P = P;
  h->nsecchi = nsecchi;
  h->ncollisioni = 0;
  h->carico = 0;

  /* Inizializziamo la tavola */
  h->tavola = (secchio *)malloc(sizeof(secchio)*nsecchi);

  for(i = 0; i < h->nsecchi; i++ ) 
	h->tavola[i].occupati = 0;
}

/* Due funzioni di hash differenti per le stringhe. */
unsigned int hash1(char *str)
{
	unsigned int hash = 5381;
	while(*str!='\0') {
	        int c = *str;
                /* hash = hash*33 + c */
                hash = ((hash << 5) + hash) + c;
                str++;
        }
	return hash;
}

unsigned int hash2(char *key)
{
    unsigned int hash = 0;
 
    while(*key != '\0')
    {
        hash += *key++;
        hash += (hash << 10);
        hash ^= (hash >> 6);
    }
    hash += (hash << 3);
    hash ^= (hash >> 11);
    hash += (hash << 15);
    return hash;
}


void calcola( hash *h, char *chiave , int *risultati)
{
   risultati[0] = hash1(chiave) % h->nsecchi;
   risultati[1] = hash2(chiave) % h->nsecchi;
}

void scambia ( chiave_val *x, chiave_val *y )
{
   chiave_val tmp = *x;
   *x = *y;
   *y = tmp;
}

chiave_val * aggiungi ( hash *h, chiave_val elemento, int secchio)
{
  int i;

  if ( h->tavola[secchio].occupati == h->P ) 
	return NULL; /* Pieno ;-( */

  if ( h->tavola[secchio].occupati == 0 ) /* Vuoto! */
      h->tavola[secchio].contenuto = (chiave_val *)malloc(sizeof(chiave_val) * h->P);      

  i = h->tavola[secchio].occupati;
  h->tavola[secchio].occupati++;

  h->tavola[secchio].contenuto [ i ] = elemento;
  return &h->tavola[secchio].contenuto[i];

}

chiave_val *cerca ( hash *h, char* chiave)
{
   int hx[2], i, j;
   calcola ( h, chiave, hx );

   for ( i = 0; i < 2; i++ ) 
     for ( j = 0; j <  h->tavola[ hx[i] ].occupati; j++)
	if ( !strcmp( h->tavola[ hx[i] ].contenuto[j].chiave, chiave ) )
		return &h->tavola[ hx[i] ].contenuto[j];

   return NULL;
}



/* Inserimento nella tavola di un elemento.
 * Restituisce il puntatore alla struttura
 * chiave_val in cui viene inserito o NULL
 * se l'inserimento e' fallito.
 */
chiave_val *inserisci_o_trova ( hash *h, char *chiave )
{
  int i,j,k;
  int pos; /* Elemento corrente e posizione corrente. */
  int hx[2];  /* Gli hash dell'elemento corrente. */
  chiave_val *risultato = NULL;
  chiave_val x;

  /* Prima di tutto controlliamo che non ci sia. */
  if ( ( risultato = cerca (h, chiave) ) != NULL )
	return risultato;
  
  x.chiave = strdup(chiave); 
  x.frequenza = 0;

  calcola(h, x.chiave, hx);
  pos = hx[0]; /* x viene sicuramente inserito in questo posto. */
  risultato = aggiungi ( h, x, pos);
  h->carico++;
  if ( risultato == NULL ) /* Non c'era spazio! Leviamo qualcuno. */
  {
     h->ncollisioni++;
     risultato = & h->tavola[pos].contenuto[0];
     scambia ( &x, risultato );
  } else 
     return risultato;


  /* Proviamo a inserire fino al numero massimo di secchi. */
  for ( i = 0; i < h->nsecchi; i++ ) 
  {
      /* Ora cerchiamo la prossima posizione da esaminare. */
      calcola ( h, x.chiave, hx ) ;
      if ( pos == hx[0] ) pos = hx[1]; 
      else pos = hx[0];

      if ( aggiungi ( h, x, pos ) != NULL ) 
	break;
	
      h->ncollisioni++;

      /* No, dobbiamo scambiare. Decidiamo di farlo col primo. */
      scambia( &x, &h->tavola[pos].contenuto[0]);
  }

  if ( i == h->nsecchi )
	return NULL;

  return risultato;
}

void stampa(hash *h)
{
  int i, j; 

  for ( i = 0; i < h->nsecchi; i++ ) /* Per ogni secchio. */
   for ( j = 0; j < h->tavola[i].occupati; j++ ) 
     printf("%s %d\n", h->tavola[i].contenuto[j].chiave, h->tavola[i].contenuto[j].frequenza);
}

int main()
{
  int N, D, M, univoci, i;
  char stringa[1024];
  chiave_val *c;

  hash h;

  scanf("%d %d %d", &N, &D, &M);  

  inizializza(&h, N, D); 
  univoci = 0;
  for ( i = 0; i < M; i++ ) 
  {
      scanf("%s", stringa);
      c = inserisci_o_trova ( &h, stringa );

      if ( c == NULL ) { 
        univoci = 0;
        for ( i = 0; i < h.nsecchi; i++ ) 
           printf("%d %d\n", i, h.tavola[i].occupati);

        return; 
      }

      if ( c->frequenza == 0 ) univoci++;
      c->frequenza++;
  }

  printf("%d\n", univoci);
  stampa(&h);
   
}

